from os_android_launcher_creator.bp import _launcher_creator as bp
from typing import Callable, Any, Iterable
from os_android_launcher_creator import AppObj

from typing import List
##################################################################################
#
# This module aim is to create a launcher icon for a given app using
# Android Studio automation
#
##################################################################################

screen_width = False
screen_height = False


def create_launcher_icons(custom_android_project_path,
                          app_list: List[AppObj],
                          shortcut_keys_to_open_image_asset= ['shift', 'b'],
                          launcher_resize_percent=50,
                          launcher_background_color_hex='#ffffff'):
    """
    Will create a launcher icons from a given icon
    NOTICE: the automation need to run in an open project in Android Studio so make sure you open one to work on!

    Args:
        custom_android_project_path: the current open Android Studio project
        app_list: a list containing the AppObj. Every AppObj contains the path to the launcher and and a path to the destination files to be copied
        shortcut_keys_to_open_image_asset: the list of buttons to hold together in order to open the Image Asset in Android Studio
        launcher_resize_percent: the resize percents of the icon to fit the frame in the editor
        launcher_background_color_hex: the color of the background of the icons
    """
    inst = bp.LauncherCreatorBP(custom_android_project_path, app_list, shortcut_keys_to_open_image_asset,  launcher_resize_percent, launcher_background_color_hex)
    inst.create_launcher_icons()



import os
from os_file_handler import file_handler as fh
from os_xml_handler import xml_handler as xh
from os_tools import tools
from os_android_launcher_creator import launcher_creator

IS_APP_SMART = 'true'


def start():
    apps_dir = '/Users/home/Google Drive/Remotes/Android/osfunapps_updates/old_apps/work/test_batch'

    app_dirs = fh.get_dir_content(apps_dir, collect_files=False, collect_dirs=True)
    done_dir = '/Users/home/Google Drive/Remotes/Android/osfunapps_updates/old_apps/work/done'

    apps = []
    for app_dir in app_dirs:
        # launcher logo
        icon_path = os.path.join(app_dir, 'logo.png')
        output_path = os.path.join(app_dir, 'files', 'launcher')
        apps.append(launcher_creator.AppObj(icon_path, output_path))

    launcher_creator.create_launcher_icons('/Users/home/Programming/android/Remotes/Projects/GeneralRemoteAndroid', apps, ['shift', 'b'], 65)

    # # done
    # dst_app_dir = os.path.join(done_dir, fh.get_dir_name(app_dir))
    # fh.copy_dir(app_dir, dst_app_dir)
    # fh.remove_dir(app_dir)


def is_ad_node_exists(parent_node, att_name_value):
    return xh.find_all_nodes(parent_node, 'ad', 'name', att_name_value)


if __name__ == "__main__":
    start()



from distutils.core import setup

setup(
    name='os_android_launcher_creator',
    packages=['os_android_launcher_creator', 'os_android_launcher_creator.bp'],
    version='1.05',  # Start with a small number and increase it with every change you make
    license='MIT',  # Chose a license from here: https://help.github.com/articles/licensing-a-repository
    description='This module will create an app launcher (logo) using Android Studio in all sizes and '
                'shapes (web, round) and in all dpi (mipmap-anydpi-v26, mipmap-hdpi, mipmap-mdpi, '
                'mipmap-xhdpi, mipmap-xxhdpi, mipmap-xxxhdpi, values)',  # Give a short description about your library
    author='Oz Shabat',  # Type in your name
    author_email='admin@os-apps.com',  # Type in your E-Mail
    url='https://github.com/osfunapps/os_android_launcher_creator-py',  # Provide either the link to your github or to your website
    keywords=['python', 'osfunapps', 'osapps', 'automation', 'logo', 'app launcher'],  # Keywords that define your package best
    install_requires=['pyautogui', 'os_tools', 'os_file_handler'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',  # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package

        'Intended Audience :: Developers',  # Define that your audience are developers
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',  # Again, pick a license

        'Programming Language :: Python :: 3',  # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)




import os
import os_tools.logger_handler as lh
import os_tools.tools as tools
import os_file_handler.file_handler as fh
import time
import pyautogui


##################################################################################
#
# just a bp for the launcher creator
#
##################################################################################


def paste():
    from sys import platform
    if platform == "darwin":
        hot_key(['command', 'v'])
    else:
        hot_key(['ctrl', 'v'])


def select_all():
    from sys import platform
    if platform == "darwin":
        hot_key(['command', 'a'])
    else:
        hot_key(['ctrl', 'a'])


def hot_key(btn_list):
    pyautogui.keyDown(btn_list[0])

    time.sleep(1)
    for i in range(1, len(btn_list)):
        pyautogui.keyDown(btn_list[i])

    time.sleep(1)
    for i in range(1, len(btn_list)):
        pyautogui.keyUp(btn_list[i])

    time.sleep(1)
    pyautogui.keyUp(btn_list[0])


class LauncherCreatorBP:
    STATIC_MAIN_RELATIVE_PATH = os.path.join('app', 'src')
    SECS_COOLDOWN_UNTIL_PROJECT_OPEN = 4

    def __init__(self, custom_android_project_path, icon_files_list, output_path, shortcut_keys_to_open_image_asset, launcher_resize_percent, launcher_background_color_hex, delete_icon_after_done):
        self.custom_android_project_path = custom_android_project_path
        self.icon_files_list = icon_files_list
        self.output_path = output_path
        self.shortcut_keys_to_open_image_asset = shortcut_keys_to_open_image_asset
        self.launcher_resize_percent = launcher_resize_percent
        self.launcher_background_color_hex = launcher_background_color_hex
        self.logger = lh.Logger(__file__)  # build the logger
        self.main_path = os.path.join(custom_android_project_path, self.STATIC_MAIN_RELATIVE_PATH)
        self.delete_icon_after_done = delete_icon_after_done

    def create_launcher_icons(self):

        self.logger.info('-------------------------------')
        self.logger.info(f'Preparing to work on {str(len(self.icon_files_list))} icons')
        time.sleep(1)
        tools.ask_for_input('Please open your custom project in Android Studio and inform me when you are done [done]')

        self.logger.info(f'Now go back to the project and wait {str(self.SECS_COOLDOWN_UNTIL_PROJECT_OPEN)} seconds...')
        time.sleep(self.SECS_COOLDOWN_UNTIL_PROJECT_OPEN)

        for i in range(0, len(self.icon_files_list)):
            self.logger.info('-----------------------------------------')
            self.logger.info(f'working on: {fh.get_file_name_from_path(self.icon_files_list[i], False)}')
            res = self.run_cycle(self.icon_files_list[i])
            if res:
                self.logger.info('done!')
            else:
                tools.ask_for_input("ERROR: It seems like the process failed for " + self.icon_files_list[i] + ". Let me know when you are ready to run on the same file [Enter]")
                i -= 1

    def run_cycle(self, app_obj):

        import random
        launcher_name = f'launcher_{str(random.randint(0, 99999))}'

        self.logger.info('old files removed. waiting a few seconds for the system to update itself')
        time.sleep(2)

        # open image assets
        hot_key(self.shortcut_keys_to_open_image_asset)

        time.sleep(1)
        # navigate to path
        self.logger.info('go to path')
        pyautogui.press('tab', 6, 0.1)

        self.logger.info('paste path')
        time.sleep(1)
        select_all()
        tools.copy_to_clipboard(app_obj.output_path)
        paste()
        time.sleep(1)

        self.logger.info('disabling trim')
        pyautogui.press('tab', 3, 0.1)
        pyautogui.press('space')

        self.logger.info('setting resize')
        pyautogui.press('tab', 2, 0.1)
        pyautogui.press('pagedown', presses=5, interval=0.1)
        time.sleep(1)
        pyautogui.press('right', presses=self.launcher_resize_percent, interval=0.1)

        time.sleep(3.5)
        self.logger.info('navigate to background')
        pyautogui.press('tab', 9, 0.1)
        pyautogui.press('right')

        time.sleep(1)

        self.logger.info('setting background color')
        pyautogui.press('tab', 3, 0.1)
        pyautogui.press('space')

        self.logger.info('clicking and setting color')
        pyautogui.press('tab', 3, 0.1)
        pyautogui.press('space')
        color = self.launcher_background_color_hex.replace('#', '')
        select_all()
        pyautogui.write(color, interval=0.1)
        time.sleep(2)
        pyautogui.press('enter')

        self.logger.info('clicking next')
        pyautogui.press('enter')

        time.sleep(1)
        self.logger.info('clicking finish')
        pyautogui.press('enter')

        self.logger.info('gathering files...')
        time.sleep(8)

        # launcher file continue....
        launcher_made_files = fh.search_file(self.main_path, prefix=launcher_name, recursive=True)

        for launcher_file in launcher_made_files:
            rel_launcher_path = launcher_file.replace(self.main_path, '')
            dst_launcher_file = os.path.join(app_obj.icon_path, rel_launcher_path)
            fh.copy_file(launcher_file, dst_launcher_file)

        fh.remove_files(launcher_made_files)

        # fh.remove_files(launcher_icons)
        # fh.copy_dir(launchers_path, project_main)

        # fh.create_dir(self.output_path)

        # # copy all of the icon files to the path, ony by one
        # icon_file_name_no_ext = fh.get_file_name_from_path(icon_file, False)
        # output_main_dir = os.path.join(self.output_path, icon_file_name_no_ext)
        # if fh.is_dir_exists(output_main_dir):
        #     file_over_write_extension = fh.get_extension_from_file(icon_file).replace('.', '_')
        #     icon_file_name_no_ext += file_over_write_extension
        #     output_main_dir = os.path.join(self.output_path, icon_file_name_no_ext)
        #     self.logger.warning(f"icon file with the name '{icon_file_name_no_ext}' already exists. Creating a unique directory: {output_main_dir} to prevent overwrite!")
        # fh.create_dir(output_main_dir)
        #
        # # if no launcher made files, it means that for some reason the process failed
        # if not launcher_made_files:
        #     return False
        #
        # self.logger.info('saving files...')
        # for launcher_file in launcher_made_files:
        #     curr_launcher_dir = fh.get_parent_path(launcher_file)
        #     src_idx = curr_launcher_dir.find('src/')
        #     curr_launcher_dir = curr_launcher_dir[src_idx + 4:]
        #     curr_launcher_dir_path = os.path.join(self.output_path, icon_file_name_no_ext, curr_launcher_dir)
        #     fh.create_dir(curr_launcher_dir_path)
        #     launcher_file_name_with_ext = fh.get_file_name_from_path(launcher_file)
        #     icon_file_dst = os.path.join(curr_launcher_dir_path, launcher_file_name_with_ext)
        #     fh.copy_file(launcher_file, icon_file_dst)

        # if self.delete_icon_after_done:
        #     fh.remove_file(icon_file)
        self.logger.info('waiting for next cycle...')
        time.sleep(1.5)
        return True

    def clear_old_launcher_files(self):
        self.logger.info('removing all old launcher made files...')
        launcher_made_files = fh.search_file(self.main_path, None, 'ic_launcher')
        fh.remove_files(launcher_made_files)
